//+------------------------------------------------------------------+
//|                                                        Gauss.mq5 |
//|                              Copyright  2015, Vladimir Karputov |
//|                                           http://wmua.ru/slesar/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2015, Vladimir Karputov"
#property link      "http://wmua.ru/slesar/"
#property version   "1.12"
#property description "Blur image using a Gaussian filter"
#property description "Alglib library is required"
#property script_show_inputs
#include <Canvas\Canvas.mqh>
#include <Math\Alglib\alglib.mqh>
//--- input
input uint  radius=2;   // radius blur
//---
int         NNodes;     // 
//+------------------------------------------------------------------+
//| version 1.10:                                                    |
//|   New: horizontal and vertical blur                              |
//| version 1.11:                                                    |
//|   New: application of the Gauss formulas                         |
//| version 1.12:                                                    |
//|   Fix: a little increased speed                                  |
//|   Fix: removed artifacts around the perimeter                    |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//--- Remember the initial value
   uint start=GetTickCount();

   NNodes=(int)radius*2+1;
//--- creating a resource
   ResetLastError();
   if(!ResourceCreate("res","\\Files\\Images\\screen.bmp"))
     {
      Print("Error creating a resource ",GetLastError());
      return;
     }
   uint res_data[];                    // array for receiving data from the graphical resource
   uint res_width=0;                   // width of the resource
   uint res_height=0;                  // height of the resource
//+------------------------------------------------------------------+
//| reads data from the graphical resource                           |
//+------------------------------------------------------------------+
   ResetLastError();
   if(!ResourceReadImage("::res",res_data,res_width,res_height))
     {
      Print("Error reads data from the graphical resource ",GetLastError());
      Print("attempt number two");
      //--- attempt number two: now the picture width and height are known
      ResetLastError();
      if(!ResourceReadImage("::res",res_data,res_width,res_height))
        {
         Print("Error reads data from the graphical resource ",GetLastError());
         return;
        }
     }
//---
   if(radius>=res_width/2 || radius>=res_height/2)
     {
      Print("Error! Image size is very small or very large blur");
      return;
     }
//+------------------------------------------------------------------+
//| decomposition of pictures on the components r, g, b              |
//+------------------------------------------------------------------+
   int      size=ArraySize(res_data);
   uint     res_blur_hor[];               // temp array for receiving data from the graphical resource
   uint     res_blur_ver[];               // temp array for receiving data from the graphical resource
   if(ArrayResize(res_blur_hor,size)==-1)
     {
      Print("Error setting the new size of an array of \"res_blur_hor\"");
      return;
     }

   if(ArrayResize(res_blur_ver,size)==-1)
     {
      Print("Error setting the new size of an array of \"res_blur_ver\"");
      return;
     }

   uint     clr_temp;                        // color in ARGB format
   uchar    r_data[],g_data[],b_data[];      // arrays of colors R, G, B
   uchar    r1_data[],g1_data[],b1_data[];   // arrays of colors R, G, B

   if(ArrayResize(r_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"r_data\"");
      return;
     }

   if(ArrayResize(g_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"g_data\"");
      return;
     }

   if(ArrayResize(b_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"b_data\"");
      return;
     }

   if(ArrayResize(r1_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"r1_data\"");
      return;
     }

   if(ArrayResize(g1_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"g1_data\"");
      return;
     }

   if(ArrayResize(b1_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"b1_data\"");
      return;
     }
//---
   double weights[];                      // array of weights
   GetQuadratureWeights(1,NNodes,weights);

   for(int i=0;i<size;i++)
     {
      clr_temp=res_data[i];
      r_data[i]=GETRGBR(clr_temp);
      g_data[i]=GETRGBG(clr_temp);
      b_data[i]=GETRGBB(clr_temp);
     }
//+------------------------------------------------------------------+
//| blur horizontal (axis X)                                         |
//+------------------------------------------------------------------+
   uint XY;                               // pixel coordinate in the array
   double   r_temp=0.0,g_temp=0.0,b_temp=0.0;
   int      coef=0;
   int      j=NNodes/2;
   for(uint Y=0;Y<res_height;Y++)
     {
      for(uint X=radius;X<res_width-radius;X++)
        {
         XY=Y*res_width+X;
         r_temp=0.0; g_temp=0.0; b_temp=0.0;
         coef=0;
         j=NNodes/2;
         for(int i=-1*j;i<j+1;i=i+1)
           {
            r_temp=r_temp+r_data[XY+i]*weights[coef];
            g_temp=g_temp+g_data[XY+i]*weights[coef];
            b_temp=b_temp+b_data[XY+i]*weights[coef];
            coef=coef+1;
           }
         r_data[XY]=(uchar)MathRound(r_temp);
         g_data[XY]=(uchar)MathRound(g_temp);
         b_data[XY]=(uchar)MathRound(b_temp);
        }
      //--- remove artifacts on the left
      for(uint x=0;x<radius;x++)
        {
         XY=Y*res_width+x;
         r_data[XY]=r_data[Y*res_width+radius];
         g_data[XY]=g_data[Y*res_width+radius];
         b_data[XY]=b_data[Y*res_width+radius];
        }
      //--- remove artifacts on the right
      for(uint x=res_width-radius;x<res_width;x++)
        {
         XY=Y*res_width+x;
         r_data[XY]=r_data[(Y+1)*res_width-radius-1];
         g_data[XY]=g_data[(Y+1)*res_width-radius-1];
         b_data[XY]=b_data[(Y+1)*res_width-radius-1];
        }
     }
//+------------------------------------------------------------------+
//| blur vertically (axis Y)                                         |
//+------------------------------------------------------------------+
   int dxdy=0;
   for(uint X=0;X<res_width;X++)
     {
      for(uint Y=radius;Y<res_height-radius;Y++)
        {
         XY=Y*res_width+X;
         r_temp=0.0; g_temp=0.0; b_temp=0.0;
         coef=0;
         j=NNodes/2;
         for(int i=-1*j;i<j+1;i=i+1)
           {
            dxdy=i*(int)res_width;
            r_temp=r_temp+r_data[XY+dxdy]*weights[coef];
            g_temp=g_temp+g_data[XY+dxdy]*weights[coef];
            b_temp=b_temp+b_data[XY+dxdy]*weights[coef];
            coef=coef+1;
           }
         r1_data[XY]=(uchar)MathRound(r_temp);
         g1_data[XY]=(uchar)MathRound(g_temp);
         b1_data[XY]=(uchar)MathRound(b_temp);
        }
      //--- remove artifacts on the top
      for(uint y=0;y<radius;y++)
        {
         XY=y*res_width+X;
         r1_data[XY]=r1_data[X+radius*res_width];
         g1_data[XY]=g1_data[X+radius*res_width];
         b1_data[XY]=b1_data[X+radius*res_width];
        }
      //--- remove artifacts on the down
      for(uint y=res_height-radius;y<res_height;y++)
        {
         XY=y*res_width+X;
         r1_data[XY]=r1_data[X+(res_height-1-radius)*res_width];
         g1_data[XY]=g1_data[X+(res_height-1-radius)*res_width];
         b1_data[XY]=b1_data[X+(res_height-1-radius)*res_width];
        }
     }
//---
   for(int i=0;i<size;i++)
     {
      clr_temp=XRGB(r1_data[i],g1_data[i],b1_data[i]);
      res_blur_ver[i]=clr_temp;
     }
//---
   ResetLastError();
   if(!ResourceCreate("res_a",res_blur_ver,res_width,res_height,0,0,0,COLOR_FORMAT_ARGB_NORMALIZE))
     {
      Print("Error creating a resource \"res_a\"",GetLastError());
      return;
     }
   ResetLastError();
   if(!ResourceSave("::res_a","\\Images\\screen1.bmp"))
     {
      Print("Error save resourse \"res_a\"",GetLastError());
      return;
     }
//---
   ResourceFree("res");
   ResourceFree("res_a");
//--- Get the spent time in milliseconds
   uint time=GetTickCount()-start;
//--- Output a message to the Experts journal
   PrintFormat("Blur image using a Gaussian filter took %d ms",time);
  }
//+------------------------------------------------------------------+
//| Gets array of quadrature weights                                 |
//+------------------------------------------------------------------+
void GetQuadratureWeights(const double mu0,const int n,double &w[])
  {
   CAlglib alglib;
   double      alp[];
   double      bet[];
   ArrayResize(alp,n);
   ArrayResize(bet,n);
   ArrayInitialize(alp,1);
   ArrayInitialize(bet,1);

   double      out_x[];
   int         inf;

   alglib.GQGenerateRec(alp,bet,mu0,n,inf,out_x,w);
  }
//+------------------------------------------------------------------+
